@@ -18,11 +18,14 @@ Config:
             List of groups (plugs and channels) to process public commands in.
         hooks (str list):
             List of hooks to enable commands for.
+        identify ((str, str list) dict):
+            Mapping of identity providers and roles, used to restrict command access.
+            
+            If non-empty, users must be identified to at least one of the given providers.  The
+            value in the mapping can be a list of roles, of which the user must hold at least one.
+            With no roles given, any user identified by the provider will be accepted.
         sets (str list):
             List of command sets to enable.
-        admins ((str, str list) dict):
-            Users authorised to execute administrative commands, a mapping of network identifiers
-            to lists of user identifiers.
 
 The binding works by making commands exposed by all listed hooks available to all listed channels,
 and to the private channels of all listed plugs.  Note that the channels need not belong to any
@@ -89,20 +92,6 @@ class CommandScope(Enum):
     shared = 2
 
 
-class CommandRole(Enum):
-    """
-    Constants representing the types of users a command is available in.
-
-    Attributes:
-        anyone:
-            All configured channels.
-        admin:
-            Only authorised users in the command group.
-    """
-    anyone = 0
-    admin = 1
-
-
 @immp.pretty_str
 class BoundCommand:
     """
@@ -116,9 +105,9 @@ class BoundCommand:
         self.hook = hook
         self.cmd = cmd
 
-    def applicable(self, channel, user, private, admin):
+    def applicable(self, channel, user, private):
         """
-        Test the availability of the current command based on the scope and role.
+        Test the availability of the current command based on the scope.
 
         Args:
             channel (.Channel):
@@ -127,8 +116,6 @@ class BoundCommand:
                 Author of the message to trigger the command.
             private (bool):
                 Result of :meth:`.Channel.is_private`.
-            admin (bool):
-                ``True`` if the author is defined as an admin of this :class:`.CommandHook`.
 
         Returns:
             bool:
@@ -138,8 +125,6 @@ class BoundCommand:
             return False
         elif self.scope == CommandScope.shared and private:
             return False
-        elif self.role == CommandRole.admin and not admin:
-            return False
         elif self.cmd.test:
             return self.cmd.test(self.hook, channel, user)
         else:
@@ -199,8 +184,6 @@ class BaseCommand:
             Parse mode for the command arguments.
         scope (.CommandScope):
             Accessibility of this command for the different channel types.
-        role (.CommandRole):
-            Accessibility of this command for different users.
         test (method):
             Additional predicate that can enable or disable a command based on hook state.
         sync_aware (bool):
@@ -213,12 +196,11 @@ class BaseCommand:
             Human-readable summary of the arguments required by the method.
     """
 
-    def __init__(self, fn, parser=CommandParser.spaces, scope=CommandScope.anywhere,
-                 role=CommandRole.anyone, test=None, sync_aware=False):
+    def __init__(self, fn, parser=CommandParser.spaces, scope=CommandScope.anywhere, test=None,
+                 sync_aware=False):
         self.fn = fn
         self.parser = parser
         self.scope = scope
-        self.role = role
         self.test = test
         self.sync_aware = sync_aware
         # Only positional arguments are produced by splitting the input, there are no keywords.
@@ -328,8 +310,7 @@ class BaseCommand:
             .FullCommand:
                 Fully-qualified instance of this base command.
         """
-        return FullCommand(name, self.fn, self.parser, self.scope, self.role, self.test,
-                           self.sync_aware, args)
+        return FullCommand(name, self.fn, self.parser, self.scope, self.test, self.sync_aware, args)
 
     def __get__(self, instance, owner):
         return BoundCommand(instance, self) if instance else self
@@ -341,8 +322,7 @@ class BaseCommand:
         return hash(self.fn)
 
     def __repr__(self):
-        return "<{}: {}, {} {}>".format(self.__class__.__name__, self.scope.name,
-                                        self.role.name, self.fn)
+        return "<{}: {} {}>".format(self.__class__.__name__, self.scope.name, self.fn)
 
 
 class FullCommand(BaseCommand):
@@ -357,8 +337,8 @@ class FullCommand(BaseCommand):
     """
 
     def __init__(self, name, fn, parser=CommandParser.spaces, scope=CommandScope.anywhere,
-                 role=CommandRole.anyone, test=None, sync_aware=False, fixed_args=()):
-        super().__init__(fn, parser, scope, role, test, sync_aware)
+                 test=None, sync_aware=False, fixed_args=()):
+        super().__init__(fn, parser, scope, test, sync_aware)
         self.name = name.lower()
         self.fixed_args = fixed_args
         if len(self.fixed_args) != self.fixed:
@@ -375,12 +355,12 @@ class FullCommand(BaseCommand):
         return hash((super().__hash__(), self.name))
 
     def __repr__(self):
-        return "<{}: {} @ {}, {} {}>".format(self.__class__.__name__, self.name, self.scope.name,
-                                             self.role.name, self.fn)
+        return "<{}: {} @ {} {}>".format(self.__class__.__name__, self.name,
+                                         self.scope.name, self.fn)
 
 
-def command(name=None, parser=CommandParser.spaces, scope=CommandScope.anywhere,
-            role=CommandRole.anyone, test=None, sync_aware=False):
+def command(name=None, parser=CommandParser.spaces, scope=CommandScope.anywhere, test=None,
+            sync_aware=False):
     """
     Decorator: mark up a hook method as a command.
 
@@ -400,8 +380,6 @@ def command(name=None, parser=CommandParser.spaces, scope=CommandScope.anywhere,
             Parse mode for the command arguments.
         scope (.CommandScope):
             Accessibility of this command for the different channel types.
-        role (.CommandRole):
-            Accessibility of this command for different users.
         test (method):
             Additional predicate that can enable or disable a command based on hook state.
         sync_aware (bool):
@@ -410,7 +388,7 @@ def command(name=None, parser=CommandParser.spaces, scope=CommandScope.anywhere,
             sync channel.  See :meth:`.SyncPlug.sync_for` for resolving this to a virtual channel.
     """
     def wrap(fn):
-        args = (fn, parser, scope, role, test, sync_aware)
+        args = (fn, parser, scope, test, sync_aware)
         return FullCommand(name, *args) if name else BaseCommand(*args)
     return wrap
 
@@ -446,8 +424,8 @@ class CommandHook(immp.Hook):
                           immp.Optional("sets", dict): {str: {str: [str]}},
                           "mapping": {str: {immp.Optional("groups", list): [str],
                                             immp.Optional("hooks", list): [str],
-                                            immp.Optional("sets", list): [str],
-                                            immp.Optional("admins", dict): {str: [str]}}}})
+                                            immp.Optional("identify", dict): {str: [str]},
+                                            immp.Optional("sets", list): [str]}}})
 
     def __init__(self, name, config, host):
         super().__init__(name, config, host)
@@ -474,14 +452,13 @@ class CommandHook(immp.Hook):
 
     def _mapping_cmds(self, mapping, channel, user, private):
         cmdgroup = set()
-        admin = user.plug and user.id in (mapping["admins"].get(user.plug.name) or [])
         for name in mapping["hooks"]:
             cmdgroup.update(set(self.discover(self.host.hooks[name]).values()))
         for label in mapping["sets"]:
             for name, cmdset in self.config["sets"][label].items():
                 discovered = self.discover(self.host.hooks[name])
                 cmdgroup.update(set(discovered[cmd] for cmd in cmdset))
-        return {cmd for cmd in cmdgroup if cmd.applicable(channel, user, private, admin)}
+        return {cmd for cmd in cmdgroup if cmd.applicable(channel, user, private)}
 
     async def commands(self, channel, user):
         """
@@ -507,9 +484,30 @@ class CommandHook(immp.Hook):
             plug = None
             private = await channel.is_private()
         mappings = []
-        for mapping in self.config["mapping"].values():
-            for label in mapping["groups"]:
-                group = self.host.groups[label]
+        identities = {}
+        for label, mapping in self.config["mapping"].items():
+            providers = mapping["identify"]
+            if providers:
+                for name, roles in providers.items():
+                    if name not in identities:
+                        try:
+                            provider = self.host.hooks[name]
+                            identities[name] = await provider.identity_from_user(user)
+                        except Exception:
+                            log.exception("Exception retrieving identity from %r for map %r",
+                                          name, label)
+                            identities[name] = None
+                            continue
+                    if not identities[name]:
+                        continue
+                    elif not roles or set(roles).intersection(identities[name].roles):
+                        log.debug("Identified %r as %r for map %r", user, identities[name], label)
+                        break
+                else:
+                    log.debug("Could not identify %r for map %r, skipping", user, label)
+                    continue
+            for name in mapping["groups"]:
+                group = self.host.groups[name]
                 if plug and group.has_plug(plug, "anywhere", "named"):
                     mappings.append(mapping)
                 elif not plug and await group.has_channel(channel):
