@@ -18,6 +18,7 @@ query basic identity information.
 """
 
 from asyncio import gather
+from collections import defaultdict
 import logging
 
 import immp
@@ -45,6 +46,8 @@ class Identity:
             Physical platform users assigned to this identity.
         roles (str list):
             Optional set of role names, if applicable to the backend.
+        profile (str):
+            URL to the identity profile page.
     """
 
     @classmethod
@@ -74,11 +77,12 @@ class Identity:
                 users.append(result)
         return users
 
-    def __init__(self, name, provider=None, links=(), roles=()):
+    def __init__(self, name, provider=None, links=(), roles=(), profile=None):
         self.name = name
         self.provider = provider
         self.links = links
         self.roles = roles
+        self.profile = profile
 
     def __eq__(self, other):
         return (isinstance(other, Identity) and
@@ -95,8 +99,14 @@ class Identity:
 class IdentityProvider:
     """
     Interface for hooks to provide identity information from a backing source.
+
+    Attributes:
+        provider_name (str):
+            Readable name of the underlying service, used when displaying info about this provider.
     """
 
+    provider_name = None
+
     async def identity_from_name(self, name):
         """
         Look up an identity by the external provider's username for them.
@@ -136,6 +146,19 @@ class WhoIsHook(immp.Hook):
 
     _identities = immp.ConfigProperty([IdentityProvider])
 
+    async def _query_all(self, query, providers=None):
+        getter = "identity_from_{}".format("user" if isinstance(query, immp.User) else "name")
+        providers = providers or self._identities
+        tasks = (getattr(provider, getter)(query) for provider in providers)
+        identities = []
+        for provider, result in zip(providers, await gather(*tasks, return_exceptions=True)):
+            if isinstance(result, Identity):
+                identities.append(result)
+            elif isinstance(result, Exception):
+                log.warning("Failed to retrieve identity from %r (%r)",
+                            provider.name, provider.provider_name, exc_info=result)
+        return identities
+
     @command("who", parser=CommandParser.none)
     async def who(self, msg, name):
         """
@@ -144,30 +167,47 @@ class WhoIsHook(immp.Hook):
         if self.config["public"]:
             providers = self._identities
         else:
-            tasks = (provider.identity_from_user(msg.user) for provider in self._identities)
-            providers = [identity.provider for identity in await gather(*tasks) if identity]
+            providers = [ident.provider for ident in await self._query_all(msg.user)]
         if providers:
-            if name[0].mention:
-                user = name[0].mention
-                tasks = (provider.identity_from_user(user) for provider in providers)
-            else:
-                tasks = (provider.identity_from_name(str(name)) for provider in providers)
-            identities = list(filter(None, await gather(*tasks)))
-            links = {link for identity in identities for link in identity.links}
-            if links:
+            identities = await self._query_all(name[0].mention or str(name), providers)
+            if identities:
+                identities.sort(key=lambda ident: ident.provider.provider_name)
+                links = defaultdict(list)
+                roles = []
+                for ident in identities:
+                    for link in ident.links:
+                        links[link].append(ident)
+                    if ident.roles:
+                        roles.append(ident)
                 text = name.clone()
+                text.prepend(immp.Segment("Info for "))
                 for segment in text:
                     segment.bold = True
-                text.append(immp.Segment(" may appear as:"))
-                for user in sorted(links, key=lambda user: user.plug.network_name):
-                    text.append(immp.Segment("\n"))
-                    text.append(immp.Segment("({}) ".format(user.plug.network_name)))
-                    if user.link:
-                        text.append(immp.Segment(user.real_name or user.username, link=user.link))
-                    elif user.real_name and user.username:
-                        text.append(immp.Segment("{} [{}]".format(user.real_name, user.username)))
-                    else:
-                        text.append(immp.Segment(user.real_name or user.username))
+                text.append(immp.Segment("\nMatching providers:"))
+                for i, ident in enumerate(identities):
+                    text.append(immp.Segment("\n{}.\t".format(i + 1)),
+                                immp.Segment(ident.provider.provider_name, link=ident.profile))
+                if links:
+                    text.append(immp.Segment("\nIdentity links:"))
+                    for user in sorted(links, key=lambda user: user.plug.network_name):
+                        text.append(immp.Segment("\n({}) ".format(user.plug.network_name)))
+                        if user.link:
+                            text.append(immp.Segment(user.real_name or user.username,
+                                                     link=user.link))
+                        elif user.real_name and user.username:
+                            text.append(immp.Segment("{} [{}]".format(user.real_name,
+                                                                      user.username)))
+                        else:
+                            text.append(immp.Segment(user.real_name or user.username))
+                        known = links[user]
+                        if known != identities:
+                            indexes = [identities.index(ident) + 1 for ident in known]
+                            text.append(immp.Segment(" {}".format(indexes)))
+                if roles:
+                    text.append(immp.Segment("\nRoles:"))
+                    for ident in roles:
+                        text.append(immp.Segment("\n({}) {}".format(ident.provider.provider_name,
+                                                                    ", ".join(ident.roles))))
             else:
                 text = "{} Name not in use".format(CROSS)
         else:
