@@ -163,6 +163,8 @@ class LocalIdentityHook(immp.Hook, AccessPredicate, IdentityProvider):
                           immp.Optional("multiple", True): bool,
                           immp.Optional("admins", dict): {str: [str]}})
 
+    provider_name = "Local"
+
     _plugs = immp.ConfigProperty([immp.Plug])
 
     def on_load(self):
@@ -250,7 +252,7 @@ class LocalIdentityHook(immp.Hook, AccessPredicate, IdentityProvider):
         """
         if not msg.user or msg.user.plug not in self._plugs:
             return
-        if self.find(msg.user):
+        if await self.find(msg.user):
             text = "{} Already identified".format(CROSS)
         else:
             pwd = IdentityGroup.hash(pwd)
